package uk.co.mrdw.photostopwatch;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.hardware.Camera;
import android.hardware.Camera.PictureCallback;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.ImageView;

public class PositionPictureCallback implements PictureCallback {
	
	private ImageView lastPhotoView;

	// Times that are waiting for their photo to be taken - will normally be empty, 
	// and don't expect to have more than 1 at the moment ...
	// using Vector as its' syncronised 
	private Vector<Long> queuedTimes = new Vector<Long>();
	private boolean mAutoRotate = true;
	
	PositionPictureCallback(ImageView lastPhotoView) {
		this.lastPhotoView = lastPhotoView;
	}

	private static final String TAG = "PositionPictureCallback";

	public void setNextTime(Long time){
		queuedTimes.add(time);
	}
	
	public void setAutoRotate(boolean autoRotate){
		mAutoRotate = autoRotate;
	}
	
	// @Override
	public void onPictureTaken(byte[] data, Camera camera) {
		Log.d(TAG, "onPictureTaken");
		long startTime = System.currentTimeMillis();
		
		byte[] copiedData = new byte[data.length];
		System.arraycopy(data, 0, copiedData, 0, data.length);// TODO not sure if this is required, but have theory that startPreview is destroying the data array ... on some devices ...
		camera.startPreview();
		TimerActivity.takingPic = false;// reset when picture saved

		long time = 0;
		if(queuedTimes.size()>0){
			time = queuedTimes.remove(0);
		}
		else{
			Log.e(TAG,"No time assigned for photo ! defaulting to system time");
			time = System.currentTimeMillis();
		}
		SaveImageTask saveImageTask = new SaveImageTask(time, lastPhotoView, mAutoRotate);
		saveImageTask.execute(copiedData);

		Log.i(TAG, "Picture callback took " + (System.currentTimeMillis() - startTime));
	}
}

class SaveImageTask extends AsyncTask<byte[], Void, Bitmap> {
	private static final String TAG = "SaveImageTask";
	private long time;
	private ImageView imageView;
	private boolean mAutoRotate;
	
	SaveImageTask(long time, ImageView imageView, boolean autoRotate) {
		this.time = time;
		this.imageView = imageView;
		mAutoRotate = autoRotate;
	}

	/**
	 * Returns bitmap for sampled display on image view
	 */
	@Override
	protected Bitmap doInBackground(byte[]... imageData) {
		Log.d(TAG,"doInBackground"+imageData);

		byte[] data = imageData[0];
		Bitmap bm = null;
		File pictureFile = Utils.getImageFile(TimerActivity.getCurrentRace(), time);// timerActivity.getOutputMediaFile(position);
		if (pictureFile == null) {
			Log.d(TAG, "Error creating media file, check storage permissions: ");
			return null;
		}

		FileOutputStream fos = null;
		try {

			BitmapFactory.Options options = new BitmapFactory.Options();
			options.inSampleSize = 8;
			bm = BitmapFactory.decodeByteArray(data, 0, data.length, options);
			if(mAutoRotate){
				boolean rotationRequired = bm.getWidth()>bm.getHeight();// not sure what to do if it's square, may need user setting to override

				Matrix matrix = new Matrix();
				if(rotationRequired){
					matrix.postRotate(90);				
				}
				bm = Bitmap.createBitmap(bm, 0, 0, bm.getWidth(), bm.getHeight(), matrix, true);				
			}
			// not rotating file saved yet - will be done on first "review" of race if required
		    fos = new FileOutputStream(pictureFile);
			fos.write(data);
			
		} catch (FileNotFoundException e) {
			Log.e(TAG, "File not found: " +pictureFile.getAbsolutePath(),e);
		} catch (IOException e) {
			Log.e(TAG, "Error accessing file: " + pictureFile.getAbsolutePath(),e);
		}
		finally{
			if(fos!=null){
				try {
					fos.close();
				} catch (IOException e) {
					Log.e(TAG, "Error closing file: " +pictureFile.getAbsolutePath(),e);
				}				
			}
			
		}
				
		return bm;
	}

	@Override
	protected void onPostExecute(Bitmap bitmap) {
		if (bitmap != null) {
			imageView.setImageBitmap(bitmap);
		}		
	}
}