package uk.co.mrdw.photostopwatch;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;

public class RaceListAdapter extends ArrayAdapter<File> {
	private static final String TAG = "RaceListAdapter";
	private final Context context;
	private final ArrayList<File> raceFolders;// using ArrayList as need remove
												// method to be supported

	public RaceListAdapter(Context context, ArrayList<File> raceFolders) {
		super(context, R.layout.position_row, raceFolders);
		this.context = context;
		this.raceFolders = raceFolders;
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		LayoutInflater inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View rowView = inflater.inflate(R.layout.race_row, parent, false);
		Button raceNameView = (Button) rowView
				.findViewById(R.id.race_row_race_name);
		final String raceDirectory = raceFolders.get(position).toString();
		final String raceName = raceFolders.get(position).getName();

		raceNameView.setText(raceFolders.get(position).getName());
		if(raceName.equals(TimerActivity.getCurrentRace().getRaceName())){
			raceNameView.setBackgroundColor(Color.GREEN);
			raceNameView.setText(raceFolders.get(position).getName()+" (running)");
		}

		
		raceNameView.setOnClickListener(new OnClickListener() {

			public void onClick(View view) {
				Log.i(TAG, "position=" + position);

				// InputStream fis;
				// BufferedReader br;
				String line;
				InputStream fis;
				BufferedReader br =null;
				try {
					fis = new FileInputStream(raceDirectory + File.separator
							+ Utils.RACE_TIMES_TEXT_FILE_NAME);
					br = new BufferedReader(
							new InputStreamReader(fis));
					ArrayList<String> times = new ArrayList<String>();
					while ((line = br.readLine()) != null) {
						times.add(line);
					}

					Intent intent = new Intent(view.getContext(),
							RaceReviewActivity.class);
					intent.putExtra(RaceData.TIMES, times);
					intent.putExtra(RaceData.RACE_NAME, raceName);
					context.startActivity(intent);

				} catch (FileNotFoundException e) {
					Log.e(TAG, "Unable to find race " + raceDirectory, e);
				} catch (IOException e) {
					Log.e(TAG, "Unable to find race " + raceDirectory, e);
				}
				finally{
					if(br!=null){
						try {
							br.close();
						} catch (IOException e) {
							Log.e(TAG, "Unable to BufferedReader for "+Utils.RACE_TIMES_TEXT_FILE_NAME +" in "+ raceDirectory, e);
						}
					}
				}

			}
		});

		Button deleteButton = (Button) rowView
				.findViewById(R.id.race_row_delete);
		// if it's the current race and is running - don't show delete button
		if(raceName.equals(TimerActivity.getCurrentRace().getRaceName())){
			deleteButton.setEnabled(false);
		}
		else{
			deleteButton.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
					Log.d(TAG, "Delete clicked" + position);
					new AlertDialog.Builder(context)
							.setIcon(android.R.drawable.ic_dialog_alert)
							.setTitle("Deleting Race")
							.setMessage(
									"Are you sure you want to delete race "
											+ raceName + " ?")
							.setPositiveButton("Yes",
									new DialogInterface.OnClickListener() {

										public void onClick(DialogInterface dialog,
												int which) {
											Log.i(TAG, "Deleting folder "
													+ raceFolders.get(position)
															.getAbsolutePath());

											boolean success = Utils
													.delete(raceFolders
															.get(position));
											File zipFile = new File(Utils
													.getRacesFolder(), raceName
													+ ".zip");
											if (zipFile.exists()) {
												Log.i(TAG, "Deleting zip file "
														+ zipFile.getAbsolutePath());
												success = zipFile.delete()
														&& success;
											}

											Log.i(TAG, raceFolders.get(position)
													.getAbsolutePath()
													+ " deleted " + success);
											if (success) {
												// this is wonky - find better way
												// of refreshing list display
												ArchiveViewActivity archiveViewActivity = (ArchiveViewActivity) context;
												archiveViewActivity.refreshList();
												raceFolders.remove(position);
												notifyDataSetChanged();
											}

										}

									}).setNegativeButton("No", null).show();
				}
			});
			
		}

		return rowView;
	}


}
