package uk.co.mrdw.photostopwatch;

import java.io.File;
import java.io.IOException;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.webkit.WebView;
import android.widget.Button;

public class RaceReviewActivity extends Activity {
	private static final String TAG = "DisplayMessageActivity";

	private static final String newLine = System.getProperty("line.separator");

	public final static String POSITION = "uk.co.mrdw.photostopwatch.POSITION";

	private static boolean viewingTimesOnly = false;// if false then viewing photos
	private RaceDataSetupTask mRaceDataSetupTask ;
	private ProgressDialog mProgressDialog ;
	
	private String raceName;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		Log.d(TAG, "creating");
		super.onCreate(savedInstanceState);
		Utils.initialiseStorageDirPath(this);

		Settings settings = new Settings();
		settings.loadFromSharedPreferences(this.getSharedPreferences(Utils.PREFS_NAME,0));

		
		Intent intent = getIntent();
		raceName = intent.getStringExtra(RaceData.RACE_NAME);
		

		setContentView(R.layout.activity_race_review);
		setReviewToggleText();
		
		if(!RaceData.isRaceReadyForReview(raceName)){
			// async task to 
			//		raceData.loadFromArchive(raceName);
			// Utils.generateResultFiles(currentRace);
			// and if RaceData.isPhotosNeedRotating(raceName); then rotate

			// if the set up task is null, or it's not running on the select race create a new one and kick it off
			// ( unlikely that it would be already running on a different race, but allows for user navigating between
			// races that require setting up)
//			if(mRaceDataSetupTask==null || (!raceName.equals( mRaceDataSetupTask.getProcessingRaceName()) )){
				mProgressDialog = new ProgressDialog(this);
				mProgressDialog.setMessage("Processing ...");
				mProgressDialog.setCancelable(false);
				mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
				mProgressDialog.setProgress(0); // set percentage completed to 0%
				mProgressDialog.show();
				mRaceDataSetupTask = new RaceDataSetupTask(this,settings);
				mRaceDataSetupTask.execute(raceName);				
//			}
			
			loadWebViewProcessing();
		}
		else{
			// go straight in 
			loadWebView();			
		}


	}
	
	
	public void updateProgress(int percent){
		
		if(mProgressDialog!=null){
			
			mProgressDialog.setProgress(percent);			
		}
	}
	
	@Override
	protected void onPause() {
		super.onPause();
	}

	/** Called when user clicks done button - goes to review page */
	public void backToTimer(View view) {
		Intent intent = new Intent(this, TimerActivity.class);
		startActivity(intent);
	}

	/** Called when user clicks archives button - goes to review page */
	public void gotoArchives(View view) {
		Intent archiveViewIntent = new Intent(this, ArchiveViewActivity.class);
		startActivity(archiveViewIntent);
	}

	/** Called when user clicks reset timet button */
	public void toggleReview(View view) {
		viewingTimesOnly = !viewingTimesOnly;
		setReviewToggleText();
		loadWebView();
	}

	private void setReviewToggleText() {
		Button toggleReviewButton = (Button) findViewById(R.id.button_toggle_review);
		if (viewingTimesOnly) {
			toggleReviewButton.setText(R.string.button_toggle_review_photos);
		} else {
			toggleReviewButton.setText(R.string.button_toggle_review_timesOnly);
		}
	}

	
	void loadWebViewProcessing(){
		WebView raceWebView = (WebView) findViewById(R.id.race_webview);
		raceWebView.loadData("<html><body><h2>Processing ....</h2></body></html>","text/html", null);
	}
	
	
	void setupComplete(){
		mProgressDialog.dismiss();
		loadWebView();
	}

	
	void loadWebView() {
		File htmlFile;
		if (viewingTimesOnly) {
			htmlFile = Utils.getRaceTimesHtmlFile(raceName);
		} else {
			htmlFile = Utils.getRaceTimesAndPhotosHtmlFile(raceName);
		}

		WebView raceWebView = (WebView) findViewById(R.id.race_webview);
		raceWebView.getSettings().setAllowFileAccess(true);
		raceWebView.getSettings().setBuiltInZoomControls(true);

		Log.d(TAG, "loading  file://" + htmlFile.getAbsolutePath());
		raceWebView.loadUrl("file://" + htmlFile.getAbsolutePath());

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.activity_race_review, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId()) {
		case android.R.id.home:
			NavUtils.navigateUpFromSameTask(this);
			return true;
		case R.id.menu_view_archives:
			Log.i(TAG, "View archives");
			{
				Intent intent = new Intent(this, ArchiveViewActivity.class);
				startActivity(intent);
			}
			return true;
		case R.id.menu_timer:
			Log.i(TAG, "Timer");
			{
				Intent intent = new Intent(this, TimerActivity.class);
				startActivity(intent);
			}
			return true;
		case R.id.menu_share_results_only:
			shareResultsOnly();
			return true;
		case R.id.menu_share_zip:
			shareZip();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	/**
	 * Most likely going to be shared as an email
	 */
	private void shareResultsOnly() {
		Log.d(TAG, "shareResultsOnly");
		RaceData race = RaceData.loadFromArchive(raceName);
		StringBuffer shareContent = new StringBuffer("Here are the results for " + raceName + newLine);
		appendResultsList(race, shareContent);
		shareContent.append(newLine + newLine + "(Photo Stopwatch  " + Utils.ANDROID_APP_URL + " )");

		Intent shareIntent = createSharingIntent("Photo Stopwatch:" + raceName, shareContent);
		startActivity(shareIntent);

	}

	
	public void sharePrepComplete(RaceData race, File zipFile){
		mProgressDialog.dismiss();
		
		StringBuffer emailContent = new StringBuffer("Please find attached zip file of photos and results for " + raceName + newLine);
		emailContent.append("Unzip and open racephotos.html to view." + newLine);
		appendResultsList(race, emailContent);
		emailContent.append(newLine + newLine + "(Photo Stopwatch " + Utils.ANDROID_APP_URL + " )");
		Intent shareIntent = createSharingIntent("Photo Stopwatch:" + raceName, emailContent);
		shareIntent.setType("application/zip");// change content to zip

		shareIntent.putExtra(Intent.EXTRA_STREAM, Uri.parse("file://" + zipFile.getAbsolutePath()));
		startActivity(Intent.createChooser(shareIntent, "Share zip ..."));
		
	}
	
	/**
	 * Most likely going to be shared as an email attachment, or by bluetooth
	 */
	private void shareZip() {
		Log.d(TAG, "shareZip");
		RaceData race = RaceData.loadFromArchive(raceName);

		File zipFile = new File(Utils.getRacesFolder(), raceName + ".zip");
		if (zipFile.exists()) {
			zipFile.delete();
		}

//		try {
			// need progress bar here ...
			mProgressDialog = new ProgressDialog(this);
			mProgressDialog.setMessage("Zipping ...");
			mProgressDialog.setCancelable(false);
			mProgressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
			mProgressDialog.setProgress(0); // set percentage completed to 0%
			mProgressDialog.show();
			
			RaceDataSharePrepTask raceDataSharePrepTask = new RaceDataSharePrepTask(this, race, zipFile);
			
			raceDataSharePrepTask.execute("");
//			Utils.zip(race.getRaceFolder(), zipFile, mProgressDialog);
//			mProgressDialog.dismiss();
//			
//			StringBuffer emailContent = new StringBuffer("Please find attached zip file of photos and results for " + raceName + newLine);
//			emailContent.append("Unzip and open racephotos.html to view." + newLine);
//			appendResultsList(race, emailContent);
//			emailContent.append(newLine + newLine + "(Photo Stopwatch " + Utils.ANDROID_APP_URL + " )");
//			Intent shareIntent = createSharingIntent("Photo Stopwatch:" + raceName, emailContent);
//			shareIntent.setType("application/zip");// change content to zip
//
//			shareIntent.putExtra(Intent.EXTRA_STREAM, Uri.parse("file://" + zipFile.getAbsolutePath()));
//			startActivity(Intent.createChooser(shareIntent, "Share zip ..."));

//		} catch (IOException e) {
//			Log.e(TAG, "Error zipping up results for " + raceName, e);
//			mProgressDialog.dismiss();
//		}

	}

////	private void zipDir(ZipOutputStream zos, File dir)
////			throws FileNotFoundException, IOException {
////		zipDir(zos,dir,"");
////	}
////	
////	private void zipDir(ZipOutputStream zos, File dir, String path)
////			throws FileNotFoundException, IOException {
////		if(!dir.isDirectory()){
////			Log.e(TAG,"Not a directory "+dir.getAbsolutePath());
////		}
////		String dirName = path+"/"+dir.getName()+"/";
////		ZipEntry entry = new ZipEntry(dirName);
////		zos.putNextEntry(entry);
////		zos.closeEntry();
////		
////		File[] files = dir.listFiles();
////		for(File file : files){
////			if(file.isDirectory()){
////				zipDir(zos,file,dirName);
////			}
////			else{
////				zipFile(zos,file,dirName);
////			}
////		}
////	}
//
//	private void zipFile(ZipOutputStream zos, File file, String dir)
//			throws FileNotFoundException, IOException {
//		byte[] bytes = getBytesFromFile(file);
//		ZipEntry entry = new ZipEntry(dir+file.getName());
//		zos.putNextEntry(entry);
//		zos.write(bytes);
//		zos.closeEntry();
//	}

	private void appendResultsList(RaceData race, StringBuffer emailContent) {
		for (int idx = 0; idx < race.getResults().size(); idx++) {
			long time = race.getResults().get(idx).first;
			boolean photoCheck = race.getResults().get(idx).second;
			if (idx == 0) {
				emailContent.append(newLine + "Started at " + race.getStartTimeString() + newLine);
			} else {
				emailContent.append(" " + idx + "   \t" + RaceData.getTimeString(time));
				if (!photoCheck) {
					emailContent.append("\t -");
				}
				emailContent.append(newLine);
			}
		}

	}

	private Intent createSharingIntent(String subject, StringBuffer emailContent) {
		Intent shareIntent = new Intent(android.content.Intent.ACTION_SEND);
		shareIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, subject);
		shareIntent.setType("plain/text");
		shareIntent.putExtra(android.content.Intent.EXTRA_TEXT, emailContent.toString());
		return shareIntent;
	}

	public ProgressDialog getProgressDialog(){
		return mProgressDialog;
	}
	
//	private byte[] getBytesFromFile(File file) throws FileNotFoundException, IOException {
//		byte[] data = new byte[(int) file.length()];
//		FileInputStream fis = null;
//		try{
//			fis = new FileInputStream(file);
//			fis.read(data);			
//		}
//		finally{
//			fis.close();
//		}
//		return data;
//	}

}


interface PercentHolder{
	void setPercent(int percent);
}

class RaceDataSharePrepTask extends AsyncTask<String,Void,String> implements PercentHolder{
	private static final String TAG = "RaceDataShareTask";
	RaceReviewActivity mRaceReviewActivity;
	private int progress = 0;
	private RaceData mRace;
	private File mZipFile;
	
	RaceDataSharePrepTask(RaceReviewActivity raceReviewActivity, RaceData race, File zipFile){
		mRaceReviewActivity = raceReviewActivity;
		mRace = race;
		mZipFile = zipFile;
	}

	public void setPercent(int percent) {
		progress = percent;
		publishProgress();		
	}

	@Override
	protected void onProgressUpdate(Void... values) {
		super.onProgressUpdate(values);
		mRaceReviewActivity.updateProgress(progress);
	}

	@Override
	protected String doInBackground(String... params) {
		try {
			Utils.zip(mRace.getRaceFolder(), mZipFile, this);
		} catch (IOException e) {
			Log.e(TAG, "Error zipping up results for " + mRace.getRaceName(), e);
			mRaceReviewActivity.getProgressDialog().dismiss();
		}
		return null;
	}

	@Override
	protected void onPostExecute(String result) {
		mRaceReviewActivity.sharePrepComplete(mRace,mZipFile);
		mRaceReviewActivity= null;
    }

}

class RaceDataSetupTask extends AsyncTask<String,Void,String> implements PercentHolder{
	private static final String TAG = "RaceDataSetupTask";
	RaceReviewActivity mRaceReviewActivity;
	private String mProcessingRaceName = null;
	private int photoRotatingPercentEstimate = 90;
	private boolean photoRotationgDone = false;
	private int overallProgress =0;
	private Settings mSettings;
	
	RaceDataSetupTask(RaceReviewActivity raceReviewActivity,Settings settings){
		mRaceReviewActivity = raceReviewActivity;
		mSettings = settings;
		photoRotatingPercentEstimate = settings.getRotateImages()==Settings.ROTATE_IMAGES_NO?0:90;
	}
	
	@Override
	protected String doInBackground(String... raceNames) {
		try{
			mProcessingRaceName = raceNames[0];
			// if setting - auto rotate && RaceData.imagesNeedRotating
			// Utils rotate race images(auto?), with percentage feed back
			if(mSettings.getRotateImages()!=Settings.ROTATE_IMAGES_NO){
				boolean checkIfNeeded = mSettings.getRotateImages()==Settings.ROTATE_IMAGES_AUTO;
				Utils.rotateImages(mProcessingRaceName, checkIfNeeded, this);
			}
			photoRotationgDone = true;
			Utils.generateResultFiles(RaceData.loadFromArchive(mProcessingRaceName),this);					
			
//			File[] imageFiles = Utils.getRaceImagesFolderWithoutCreating(mProcessingRaceName).listFiles();
//			String[] filePaths = new String[imageFiles.length];
//			for(int i=0;i<imageFiles.length;i++){
//				filePaths[i]=imageFiles[i].getAbsolutePath();
//			}
//			
//			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) {
//				
//				MediaScannerConnection.
//					scanFile(mRaceReviewActivity,
//				           filePaths,
//				           null,
//				          new MediaScannerConnection.OnScanCompletedListener(){
//					@Override
//					public void onScanCompleted(String path, Uri uri) {
//				          Log.i("ExternalStorage", "Scanned " + path + ":");
//				          Log.i("ExternalStorage", "-> uri=" + uri);
//					}
//				 });		
//			}			
			
		}
		finally{
			photoRotationgDone = true;
			mProcessingRaceName = null;
		}
		return null;
	}
	
	@Override
	protected void onPostExecute(String result) {
		mRaceReviewActivity.setupComplete();
		mRaceReviewActivity= null;
    }
	
	@Override
	protected void onProgressUpdate(Void... values) {
		super.onProgressUpdate(values);
		mRaceReviewActivity.updateProgress(overallProgress );
	}
	
	public String getProcessingRaceName(){
		return mProcessingRaceName;
	}

	public void setPercent(int percent) {
		overallProgress = percent;
		if( !photoRotationgDone ){
			overallProgress = (percent*photoRotatingPercentEstimate)/100;
		}
		else{
			overallProgress = photoRotatingPercentEstimate + (percent*(100-photoRotatingPercentEstimate)/100);
		}
		publishProgress();
		Log.d(TAG,"Updating percent "+overallProgress);
	}
	
}
