package uk.co.mrdw.photostopwatch;

import android.app.Activity;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.ToggleButton;

public class SettingsActivity  extends Activity {
	private static final String TAG = "SettingsActivity";

	private Settings settings;
	private Settings settingsOldSettings;
		
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Utils.initialiseStorageDirPath(this);

		setContentView(R.layout.activity_settings);
	}
	
	@Override
	public void onStart() {
		super.onStart();

		settings = new Settings();
		settings.loadFromSharedPreferences(this.getSharedPreferences(Utils.PREFS_NAME, 0));
		settingsOldSettings = settings.copy();
		setControls();
	}
	
	
	private void setControls(){
		
		PackageInfo pInfo;
		try {
			pInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
			TextView versionTextView = (TextView) findViewById(R.id.settingsVersionText);
			versionTextView.setText("App Version: "+pInfo.versionName);
		} catch (NameNotFoundException e) {
			Log.e(TAG,"Couldn't find app version",e);
		}
		
		RadioButton leftHandedRadio = (RadioButton) findViewById(R.id.leftHandRadioButton);
		RadioButton rightHandedRadio = (RadioButton) findViewById(R.id.rightHandRadioButton);
		leftHandedRadio.setChecked(settings.isLeftHanded());
		rightHandedRadio.setChecked(!settings.isLeftHanded());
		
		ToggleButton headsetButton = (ToggleButton) findViewById(R.id.headsetControlToggleButton);
		headsetButton.setChecked(settings.isHeadsetEnabled());

		ToggleButton volumeKeysButton = (ToggleButton) findViewById(R.id.volumeKeyControlToggleButton);
		volumeKeysButton.setChecked(settings.isVolumeKeysEnabled());

		int decimalPlaces = settings.getTimeDecimalPlaces();
		switch( decimalPlaces ){
		case 0:
			RadioButton dp0Radio = (RadioButton) findViewById(R.id.dp0RadioButton);
			dp0Radio.setChecked(true);
			break;
		case 1:
			RadioButton dp1Radio = (RadioButton) findViewById(R.id.dp1RadioButton);
			dp1Radio.setChecked(true);
			break;
		case 2:
			RadioButton dp2Radio = (RadioButton) findViewById(R.id.dp2RadioButton);
			dp2Radio.setChecked(true);
			break;
		case 3:
			RadioButton dp3Radio = (RadioButton) findViewById(R.id.dp3RadioButton);
			dp3Radio.setChecked(true);
			break;
		}
		
		ToggleButton chronoShowTenthsButton = (ToggleButton) findViewById(R.id.chronoShowTenthsToggleButton);
		chronoShowTenthsButton.setChecked(settings.isChronoShowTenths());

		ToggleButton imageSizeToggleButton = (ToggleButton) findViewById(R.id.imageSizeToggleButton);
		imageSizeToggleButton.setChecked(settings.getMinPictureHeight()!=Settings.PIC_HEIGHT_SMALL);

		int imageRotation = settings.getRotateImages();
		switch( imageRotation ){
		case Settings.ROTATE_IMAGES_AUTO:
			RadioButton autoRotateRadioButton = (RadioButton) findViewById(R.id.autoRotateRadioButton);
			autoRotateRadioButton.setChecked(true);
			break;
		case Settings.ROTATE_IMAGES_ALWAYS:
//			RadioButton alwaysRotateRadioButton = (RadioButton) findViewById(R.id.alwaysRotateRadioButton);
//			alwaysRotateRadioButton.setChecked(true);
			break;
		case Settings.ROTATE_IMAGES_NO:
			RadioButton noRotateRadioButton = (RadioButton) findViewById(R.id.noRotateRadioButton);
			noRotateRadioButton.setChecked(true);
			break;
		}
	}
	
	
	public void onHandednessRadioButtonClicked(View view) {
	    // Is the button now checked?
	    boolean checked = ((RadioButton) view).isChecked();
	    
	    // Check which radio button was clicked
	    switch(view.getId()) {
	        case R.id.leftHandRadioButton:
	        	settings.setLeftHanded(checked);
	            break;
	        case R.id.rightHandRadioButton:
	        	settings.setLeftHanded(!checked);	            	
	            break;
	    }
		saveSettings();
	}	
	
	
	public void onChronoShowTenthsToggleClicked(View view){
	    boolean tenthsEnabled = ((ToggleButton) view).isChecked();
	    settings.setChronoShowTenths(tenthsEnabled);
		saveSettings();
	}
	
	public void onDecimalPlacesRadioButtonClicked(View view) {
	    // Check which radio button was clicked
	    switch(view.getId()) {
	        case R.id.dp0RadioButton:
	        	settings.setTimeDecimalPlaces(0);
	            break;
	        case R.id.dp1RadioButton:
	        	settings.setTimeDecimalPlaces(1);
	            break;
	        case R.id.dp2RadioButton:
	        	settings.setTimeDecimalPlaces(2);
	            break;
	        case R.id.dp3RadioButton:
	        	settings.setTimeDecimalPlaces(3);
	            break;
	    }
		saveSettings();	    
	}	
	
	
	public void onRotationButtonClicked(View view) {
	    // Check which radio button was clicked
	    switch(view.getId()) {
	        case R.id.autoRotateRadioButton:
	        	settings.setRotateImages(Settings.ROTATE_IMAGES_AUTO);
	            break;
	        case R.id.noRotateRadioButton:
	        	settings.setRotateImages(Settings.ROTATE_IMAGES_NO);
	            break;
	    }
		saveSettings();
	    
	}	
	

	public void onHeadsetToggleClicked(View view) {
	    boolean headsetEnabled = ((ToggleButton) view).isChecked();
	    settings.setHeadsetEnabled(headsetEnabled);
		saveSettings();
	}	
	
	public void onVolumeKeyToggleClicked(View view) {
	    boolean volumeKeysEnabled = ((ToggleButton) view).isChecked();
	    settings.setVolumeKeysEnabled(volumeKeysEnabled);
		saveSettings();
	}	
	
	
	public void onImageSizeToggleClicked(View view) {
	    boolean imageSizeLarge = ((ToggleButton) view).isChecked();
	    if(imageSizeLarge){
	    	settings.setMinPictureHeight(Settings.PIC_HEIGHT_LARGE);
	    }
	    else{
	    	settings.setMinPictureHeight(Settings.PIC_HEIGHT_SMALL);
	    }
	    saveSettings();
	}
	
	public void settingsReset(View view){
		settings = new Settings();
		saveSettings();
		setControls();
	}
	
	public void settingsCancel(View view){
		settings = settingsOldSettings.copy();
		saveSettings();
		finish();
	}
		
	public void done(View view){
		finish();
	}
	
	public void saveSettings(){
	    RaceData.setDecimalPlaces(settings.getTimeDecimalPlaces());// TODO this is smelly - but how it works for now
		settings.saveToSharedPreferences(this.getSharedPreferences(Utils.PREFS_NAME, 0)); 		
	}

}
