package uk.co.mrdw.photostopwatch;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Pair;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class TimeListAdapter extends ArrayAdapter<Pair<Long,Boolean>> {
	private final Context context;
	private RaceData race;
	private boolean showJpgs;

	public TimeListAdapter(Context context, RaceData race) {
		this(context, race, false);
	}

	public TimeListAdapter(Context context, RaceData race, boolean showJpgs) {
		super(context, R.layout.position_row, race.getResults());
		this.context = context;
		this.race = race;
		this.showJpgs = showJpgs;
	}

	@Override
	public View getView(int idx, View convertView, ViewGroup parent) {
		int position = idx;
		LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

		View rowView;
		if (showJpgs) {
			rowView = inflater.inflate(R.layout.position_jpg_row, parent, false);
		} else {
			rowView = inflater.inflate(R.layout.position_row, parent, false);
		}

		
		TextView positionLabelView = (TextView) rowView.findViewById(R.id.position_label);
		TextView timeLabelView = (TextView) rowView.findViewById(R.id.time_label);
		// ImageView imageView = (ImageView) rowView.findViewById(R.id.icon);

		if(position==0){
			positionLabelView.setText(R.string.start_time_label);						
			
			timeLabelView.setText(race.getStartTimeString());
		}
		else{
			positionLabelView.setText(String.valueOf(position));			
			timeLabelView.setText(race.getTimeStringForPosition(position) +  (race.isPhotoCheckForPosition(position)?"":" *") );
		}
		if (showJpgs) {
			ImageView photoThumbnail = (ImageView) rowView.findViewById(R.id.photo_thumb);
			String jpgPath = Utils.getImageFile(race, position).getAbsolutePath();

			BitmapFactory.Options options = new BitmapFactory.Options();
			options.inSampleSize = 16;
			Bitmap bm = BitmapFactory.decodeFile(jpgPath, options);

			photoThumbnail.setImageBitmap(bm);

		}


		return rowView;
	}

}
