package uk.co.mrdw.photostopwatch;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import uk.co.mrdw.chronometer.Chronometer;
import uk.co.mrdw.photostopwatch.R.drawable;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Typeface;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.util.Pair;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Toast;

/**
 * Activity for the Timer - User can start the race and log finishing times Race
 * Timer is displayed, with positions and times logged so far. A small preview
 * of the camera is displayed so photo's can be captured when race is started
 * and times are logged.
 * 
 * If the user navigates away the positions and times logged so far are saved in
 * a text file.
 * 
 * @author Mike Warren 20 Aug 2012
 * 
 */
public class TimerActivity extends Activity {
	private static final String TAG = "TimerActivity";

	private static final Void VOID_PARAMS = null;

	// statics used to keep track of state
	static private RaceData currentRace = new RaceData();
	static boolean takingPic = false;// if true then the camera's busy, so don't
										// try and take photo

	private ArrayAdapter<Pair<Long, Boolean>> timesListAdapter = null;
	private Camera mCamera;
	private CameraPreview mPreview;
	private PositionPictureCallback mPictureCallback;
	private static MediaButtonReceiver mediaButtonReceiver = null;
	private boolean disabledBecauseNoSD = false;
	private Settings settings = new Settings();

	// private MediaPlayer buttonClickMp;// not using beep for now - maybe add as
										// an option?TODO

	public static RaceData getCurrentRace() {
		return currentRace;
	}

	public static void setCurrentRace(RaceData currentRace) {
		TimerActivity.currentRace = currentRace;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		Log.d(TAG,
				"creating ..................................................");
		super.onCreate(savedInstanceState);

		Utils.initialiseStorageDirPath(this);

		settings.loadFromSharedPreferences(this.getSharedPreferences(
				Utils.PREFS_NAME, 0));

		PackageInfo pInfo;
		try {
			pInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
			Utils.setVersionName(pInfo.versionName);
		} catch (NameNotFoundException e) {
			Log.e(TAG, "Couldn't find app version", e);
		}

		RaceData.setDecimalPlaces(settings.getTimeDecimalPlaces());
		populateRaceFromInstanceState(savedInstanceState);

		setContentView(R.layout.activity_timer);
//		buttonClickMp = MediaPlayer.create(this, R.raw.beep_21); TODO
		createTimerButtons();
		// arrangeLeftRightHandedView();

		final ImageView lastPhotoView = (ImageView) findViewById(R.id.last_photo);
		mPictureCallback = new PositionPictureCallback(lastPhotoView);

		ListView listView = (ListView) findViewById(R.id.results_list);
		timesListAdapter = new TimeListAdapter(this, currentRace);
		listView.setAdapter(timesListAdapter);
		scrollToLastPosition();
		Chronometer chronometer = (Chronometer) findViewById(R.id.chronometer);
		// Button startButton = (Button) findViewById(R.id.startTimeButton);
		chronometer.setBase(currentRace.getChronometerBase());
		if (currentRace.isChronometerRunning()) {
			chronometer.start();
		} else {
			chronometer.setBase(SystemClock.elapsedRealtime());
			currentRace.setChronometerBase(chronometer.getBase());
		}
		enableDisableDoneButon();
		if(takingPic){
			Log.e(TAG,"takingPic true in on create ?");
			takingPic = false;// quick fix for bug where it didn't seem to get reset ?
		}
	}

	private void enableDisableDoneButon() {
		Button doneButton = (Button) findViewById(R.id.doneButton);
		doneButton.setEnabled(currentRace.isChronometerRunning());
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.activity_timer, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId()) {
		case R.id.menu_settings:
			Log.i(TAG, "settings");
			Intent settingsIntent = new Intent(this, SettingsActivity.class);
			startActivity(settingsIntent);
			return true;
		case R.id.menu_help:
			Log.i(TAG, "Help");
			return true;
		case R.id.menu_view_archives:
			Log.i(TAG, "View archives");
			if (!disabledBecauseNoSD) {
				Intent archiveViewIntent = new Intent(this,
						ArchiveViewActivity.class);
				startActivity(archiveViewIntent);
			}
			return true;
		case R.id.menu_race_review:
			Log.i(TAG, "Race review");
			if ((!disabledBecauseNoSD) && (!currentRace.isReset())) { 
				// save html TODO - may not be place to do this, could rely
				// on RaceReview activity  to do it if it needs it ...
				// Utils.generateResultFiles(currentRace);
				Intent reviewIntent = new Intent(this, RaceReviewActivity.class);
				currentRace.populateIntent(reviewIntent);
				startActivity(reviewIntent);
			} else {
				Toast.makeText(this, R.string.toast_nothing_to_review,
						Toast.LENGTH_SHORT).show();
			}
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	private void disableMediaButtonControl() {
		if (mediaButtonReceiver != null) {
			unregisterReceiver(mediaButtonReceiver);
			mediaButtonReceiver = null;
		}
	}

	private void setMediaButtonControl() {
		if (mediaButtonReceiver != null) {
			unregisterReceiver(mediaButtonReceiver);
			mediaButtonReceiver.setTimerActivity(this);
		} else {
			mediaButtonReceiver = new MediaButtonReceiver(this);
		}
		IntentFilter filter = new IntentFilter(Intent.ACTION_MEDIA_BUTTON);
		filter.setPriority(2147483647);// android.content.IntentFilter.SYSTEM_HIGH_PRIORITY-1);//
										// TODO investigate how bad this is ...
										// http://stackoverflow.com/questions/6287116/android-registering-a-headset-button-click-with-broadcastreceiver
		registerReceiver(mediaButtonReceiver, filter);

	}


	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		if (!settings.isVolumeKeysEnabled()) {
			return super.dispatchKeyEvent(event);
		}
		int action = event.getAction();
		int keyCode = event.getKeyCode();
		switch (keyCode) {
		case KeyEvent.KEYCODE_VOLUME_UP:
			if ((event.getRepeatCount() == 0) // only want to register click, not key being held down (I.e. repeating)
					&& (action == KeyEvent.ACTION_DOWN)) {
				logTime();
			}
			return true;
		case KeyEvent.KEYCODE_VOLUME_DOWN:
			if ((event.getRepeatCount() == 0)
					&& (action == KeyEvent.ACTION_DOWN)) {
				logTime();
			}
			return true;
		default:
			return super.dispatchKeyEvent(event);
		}
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		// MenuItem swapHandsMenuItem = menu.findItem(R.id.menu_swap_hands);

		// if (leftHanded) {// if left handed operation, want option menu to
		// give right handed as option and vice versa
		// swapHandsMenuItem.setTitle(R.string.menu_right_hand);
		// } else {
		// swapHandsMenuItem.setTitle(R.string.menu_left_hand);
		// }

		// MenuItem headsetControlMenuItem =
		// menu.findItem(R.id.menu_headset_control);
		// if (allowControlByHeadsetButton) {
		// headsetControlMenuItem.setTitle(R.string.menu_headset_control_off);
		// } else {
		// headsetControlMenuItem.setTitle(R.string.menu_headset_control_on);
		// }

		return true;
	}

	private void createTimerButtons() {
		LinearLayout timerButtonsLayout = (LinearLayout) findViewById(R.id.timer_buttons_layout);
		timerButtonsLayout.removeAllViews();

		Button startTimeButton = new Button(this);
		startTimeButton.setId(R.id.startTimeButton);
		startTimeButton.setTextSize(30);
		startTimeButton.setTypeface(Typeface.DEFAULT_BOLD);
		timerButtonsLayout.addView(startTimeButton);
		startTimeButton.getLayoutParams().height = ViewGroup.LayoutParams.MATCH_PARENT;
		startTimeButton.getLayoutParams().width = ViewGroup.LayoutParams.MATCH_PARENT;
		addButtonListener(startTimeButton);
	}

	/**
	 * Works out which buttons to show depending on state - e.g. if show "time"
	 * button, or save and reset buttons
	 */
	private void arrangeTimerButtons() {
		LinearLayout timerButtonsLayout = (LinearLayout) findViewById(R.id.timer_buttons_layout);
		Button startTimeButton = (Button) findViewById(R.id.startTimeButton);

		timerButtonsLayout.removeAllViews();
		if (Utils.hasWritableStorage()) {
			if (currentRace.isReset() || currentRace.isChronometerRunning()) {
				Log.i(TAG, "currentRace.isReset() =" + currentRace.isReset()
						+ "      currentRace.isChronometerRunning()"
						+ currentRace.isChronometerRunning());
				startTimeButton.setVisibility(View.VISIBLE);
				if (currentRace.isChronometerRunning()) {
					startTimeButton.setText(R.string.button_time);
					startTimeButton
							.setBackgroundResource(R.drawable.yellow_button);
				} else {
					startTimeButton.setText(R.string.button_start);
					startTimeButton
							.setBackgroundResource(R.drawable.green_button);
				}
			} else {
				// add reset and save buttons
//				saveButton.setVisibility(View.VISIBLE);
//				resetButton.setVisibility(View.VISIBLE);
				startTimeButton.setVisibility(View.GONE);
			}
		}

		// need to add all the buttons even though some are invisible, so we can
		// find them again
		timerButtonsLayout.addView(startTimeButton);

	}


	private void resetRace() {
		Chronometer chronometer = (Chronometer) findViewById(R.id.chronometer);
		chronometer.setBase(SystemClock.elapsedRealtime());
		resetLastPhoto();
		currentRace.reset();
		timesListAdapter.notifyDataSetChanged();
		arrangeTimerButtons();
		enableDisableDoneButon();
	}

	private void arrangeLeftRightHandedView() {
		LinearLayout timerControls = (LinearLayout) findViewById(R.id.timer_controls);
		LinearLayout timerButtonsLayout = (LinearLayout) findViewById(R.id.timer_buttons_layout);

		arrangeTimerButtons();

		View cameraPreview = findViewById(R.id.camera_preview);

		if (mCamera != null) {
			mCamera.stopPreview();
		}
		timerControls.removeAllViews();

		LinearLayout resultsView = (LinearLayout) findViewById(R.id.results_view);
		View resultsListView = findViewById(R.id.results_list);
		View lastPhoto = findViewById(R.id.last_photo);
		resultsView.removeAllViews();

		if (settings.isLeftHanded()) {// swap over button and preview
			timerControls.addView(timerButtonsLayout);

			timerControls.addView(cameraPreview);
			resultsView.setGravity(Gravity.RIGHT);
			resultsView.addView(resultsListView);
			resultsView.addView(lastPhoto);
		} else {
			timerControls.addView(cameraPreview);
			timerControls.addView(timerButtonsLayout);
			// timerControls.addView(saveButton);
			// timerControls.addView(resetButton);

			resultsView.setGravity(Gravity.LEFT);
			resultsView.addView(lastPhoto);
			resultsView.addView(resultsListView);
		}
		if (mCamera != null) {
			mCamera.startPreview();
		}
	}

	// potentially needed to avoid data for current race being lost ?
	@Override
	protected void onSaveInstanceState(Bundle savedInstanceState) {
		super.onSaveInstanceState(savedInstanceState);
		Log.i(TAG, "onSaveInstanceState");
		currentRace.populateBundle(savedInstanceState);
	}

	/**
	 * All restoration done in onCreate, maybe need to implement this for task
	 * that's killed ?
	 * 
	 * @param savedInstanceState
	 */
	@Override
	public void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);
		Log.i(TAG, "onRestoreInstanceState");
	}

	private void populateRaceFromInstanceState(Bundle savedInstanceState) {
		Log.d(TAG, "populateRaceFromInstanceState " + savedInstanceState);
		if (savedInstanceState == null) {
			return;
		}
		currentRace.populateFromBunndle(savedInstanceState);
		Log.d(TAG, "populateRaceFromInstanceState " + currentRace.getRaceName());
		Log.d(TAG, "populateRaceFromInstanceState race size = "
				+ currentRace.getResults().size());
	}

	@Override
	protected void onPause() {
		getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		disableMediaButtonControl();
		releaseCamera();
		super.onPause();
	}

	@Override
	protected void onStart() {
		super.onStart();
		Log.d(TAG,
				"starting ..................................................");
		// start camera preview

	};

	@Override
	protected void onStop() {
		super.onStop();
		// stop camera preview
		Log.i(TAG, "Stopping .......................................");
	};

	/**
	 * may be coming back from settings change , so recheck settings here
	 */
	@Override
	protected void onResume() {
		Log.i(TAG, "Resuming .......................................");
		super.onResume();
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		settings.loadFromSharedPreferences(this.getSharedPreferences(
				Utils.PREFS_NAME, 0));

		mPictureCallback
				.setAutoRotate(settings.getRotateImages() == Settings.ROTATE_IMAGES_AUTO);

		Chronometer chronometer = (Chronometer) findViewById(R.id.chronometer);
		chronometer.setShowTenths(settings.isChronoShowTenths());
		if (settings.isChronoShowTenths()) {
			chronometer.setTextScaleX(0.8f);
		} else {
			chronometer.setTextScaleX(1f);
		}
		chronometer
				.refreshText(currentRace.isChronometerRunning() ? SystemClock
						.elapsedRealtime() : chronometer.getBase());

		arrangeLeftRightHandedView();

		startCamera();
		if (!Utils.hasWritableStorage()) {
			Toast.makeText(this, R.string.warn_sd_card_not_writeable,
					Toast.LENGTH_LONG).show();

			LinearLayout timerButtonsLayout = (LinearLayout) findViewById(R.id.timer_buttons_layout);
			Button startTimeButton = (Button) findViewById(R.id.startTimeButton);
			Button saveButton = (Button) findViewById(R.id.saveButton);
			Button resetButton = (Button) findViewById(R.id.resetButton);
			saveButton.setVisibility(View.GONE);
			resetButton.setVisibility(View.GONE);
			startTimeButton.setVisibility(View.GONE);

			Button errorButton = (Button) findViewById(R.id.sdErrorButton);
			if (errorButton == null) {
				errorButton = new Button(this);
				errorButton.setId(R.id.sdErrorButton);
				errorButton.setText(R.string.sd_card_not_writeable_button);
				errorButton.setOnClickListener(new OnClickListener() {

					public void onClick(View v) {
						Utils.initialiseStorageDirPath(v.getContext());
						if (Utils.hasWritableStorage()) {
							disabledBecauseNoSD = false;
							arrangeTimerButtons();
						} else {
							Toast.makeText(v.getContext(),
									R.string.warn_sd_card_not_writeable,
									Toast.LENGTH_SHORT).show();
						}
					}
				});
				timerButtonsLayout.addView(errorButton);
			}

			disabledBecauseNoSD = true;
		} else {
			if (disabledBecauseNoSD) {
				disabledBecauseNoSD = false;
				arrangeTimerButtons();
			}
		}
		if (settings.isHeadsetEnabled()) {
			this.setMediaButtonControl();
		}
	};

	@Override
	protected void onDestroy() {
		super.onDestroy();
		Log.i(TAG, "Destroying .......................................");
		releaseCamera();
	};

	private void startCamera() {
		if (checkCameraHardware(getApplicationContext())) {
			Log.i(TAG, "camera checked");

			// Create an instance of Camera
			mCamera = getCameraInstance();

			if (mCamera != null) {
				Log.i(TAG, "camera instance retrieved");
				Camera.Parameters parameters = mCamera.getParameters();
				parameters.set("rotation", 90);
				parameters.set("orientation", "portrait");
				parameters.setJpegQuality(Utils.JPEG_QUALITY);
				List<Size> sl = parameters.getSupportedPictureSizes();

				// now that you have the list of supported sizes, pick one and
				// set it back to the parameters...
				int width = parameters.getPictureSize().width;
				int height = parameters.getPictureSize().height;
				Log.d(TAG, "Picture size original width=" + width + " height="
						+ height);

				for (Size s : sl) {
					// want the smallest size > 300px (or a set in settings)
					// wide (will be height as haven't rotated yet)
					Log.d(TAG, "Picture size available width=" + s.width
							+ " height=" + s.height);
					if (s.height >= settings.getMinPictureHeight()
							&& s.height < width && (s.width != s.height)) { // also
																			// ignore
																			// if
																			// it's
																			// square
																			// format
						width = s.width;
						height = s.height;
					}
				}

				Log.d(TAG, "Setting picture size to width=" + width
						+ " height=" + height);
				parameters.setPictureSize(width, height);

				mCamera.setParameters(parameters);
				// mCamera.setDisplayOrientation(90);// only since Android 8
				setCameraOrientation(mCamera);
				LinearLayout preview = (LinearLayout) findViewById(R.id.camera_preview);
				mPreview = new CameraPreview(this, mCamera);
				preview.addView(mPreview);
			}
		} else {
			Log.e(TAG, "No Camera!");

		}
		Button captureButton = (Button) findViewById(R.id.startTimeButton);

		addButtonListener(captureButton);
	}

	@SuppressLint("NewApi")
	private void setCameraOrientation(Camera mCamera) {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) {
			mCamera.setDisplayOrientation(90);// only since Android 8
		}

		// Utils.setCameraDisplayOrientation(this, 0 , mCamera);
	}

	private void addButtonListener(Button captureButton) {
		// using touch listener, so can time as soon as press down on button
		// ("on click" happens when button released)
		captureButton.setOnTouchListener(new OnTouchListener() {
			public boolean onTouch(View v, MotionEvent event) {
				if( event.getAction()==MotionEvent.ACTION_DOWN ){
					logTime();					
				}
		     return true;
		    }

		   });

		captureButton.setOnClickListener(new View.OnClickListener() {
			// @Override
			public void onClick(View v) {
				logTime();
			}
		});
	}
	

	public void logTime() {
		long timeInMillis = startLog();
		boolean photoCheck = false;
		if (mCamera != null && !takingPic) {
			photoCheck = true;
			takingPic = true;// reset when picture saved
		}
		Pair<Long, Boolean> result = new Pair<Long, Boolean>(timeInMillis,
				photoCheck);
		timesListAdapter.add(result);// this populates list shared with current
										// race
		scrollToLastPosition();

		// get an image from the camera
		if (photoCheck) {
			mPictureCallback.setNextTime(timeInMillis);
			Log.i(TAG, "Taking pic " + timeInMillis);
			long startTime = System.currentTimeMillis();
			mCamera.takePicture(null, null, mPictureCallback);
			Log.i(TAG, "pic taken in "
					+ (System.currentTimeMillis() - startTime));
		} else {
			// buttonClickMp.start();// no photo, so beep acknowldgement

			if (!takingPic) {
				Log.i(TAG, "No camera, so no pic for position "
						+ (currentRace.getResults().size() - 1) + " time:"
						+ RaceData.getTimeString(timeInMillis));
			} else {
				resetLastPhoto();
				Log.i(TAG, "Camera busy, so no pic for position "
						+ (currentRace.getResults().size() - 1) + " time:"
						+ RaceData.getTimeString(timeInMillis));
			}
		}
		Log.i(TAG, "saving " + currentRace.getRaceName() + "     ---    "
				+ timeInMillis + "  - " + photoCheck);
		AsyncSaveResultTask asyncSaveResultTask = new AsyncSaveResultTask(
				currentRace, timeInMillis, photoCheck);
		asyncSaveResultTask.execute(VOID_PARAMS);
	}

	private void resetLastPhoto() {
		final ImageView lastPhotoView = (ImageView) findViewById(R.id.last_photo);
		lastPhotoView.setImageResource(drawable.ic_launcher);
	}

	/** Called when user clicks Start / Log button */
	public long startLog() {
		Log.d(TAG,
				"start/log ..................................................");
		Chronometer chronometer = (Chronometer) findViewById(R.id.chronometer);
		long timeInMillis = 0;
		if (currentRace.isChronometerRunning()) {
			timeInMillis = SystemClock.elapsedRealtime()
					- chronometer.getBase();
		} else {
			chronometer.setBase(SystemClock.elapsedRealtime());
			timeInMillis = System.currentTimeMillis();
			currentRace.setChronometerBase(chronometer.getBase());
			chronometer.start();
			currentRace.setChronometerRunning(true);
			Button startButton = (Button) findViewById(R.id.startTimeButton);
			startButton.setText(R.string.button_time);
			startButton.setBackgroundResource(R.drawable.yellow_button);
			currentRace.setRaceName(new SimpleDateFormat("yyyy-MM-dd_HHmm-ss",Locale.US)
					.format(new Date()));
			enableDisableDoneButon();
		}
		return timeInMillis;
	}

	@SuppressLint("NewApi")
	private void scrollToLastPosition() {
		int idx = timesListAdapter.getCount() - 1;
		ListView listView = (ListView) findViewById(R.id.results_list);
		if (listView.getFirstVisiblePosition() > idx
				|| listView.getLastVisiblePosition() <= idx) {
			if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) {
				listView.smoothScrollToPosition(idx);
			} else {
				listView.setSelection(idx);
			}
		}
		// if (idx > 2) {
		// listView.getChildAt(idx-1).setBackgroundColor(Color.CYAN);
		// if (idx > 3) {
		// listView.getChildAt(idx - 2).setBackgroundColor(Color.WHITE);
		// }
		// }
	}

	private void releaseCamera() {
		if (mCamera != null) {
			mCamera.stopPreview();
			mCamera.release(); // release the camera for other applications
			mCamera = null;
			LinearLayout preview = (LinearLayout) findViewById(R.id.camera_preview);
			preview.removeView(mPreview);
			mPreview = null;
		}
	}

	/**
	 * Called when user confirms they're done, and wants to stop the timer
	 */
	private void doneConfirmed() {
		Log.d(TAG, "doneConfirmed");
		Chronometer chronometer = (Chronometer) findViewById(R.id.chronometer);
		chronometer.stop();
		currentRace.setChronometerRunning(false);
		enableDisableDoneButon();
		Intent intent = new Intent(this, RaceReviewActivity.class);
		currentRace.populateIntent(intent);
		resetRace();
		startActivity(intent);
	}

	/** Called when user clicks done button - goes to review page */
	public void done(View view) {

		new AlertDialog.Builder(this)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setTitle("Done")
				.setMessage("Are you sure you want to stop timer?")
				.setPositiveButton("Yes",
						new DialogInterface.OnClickListener() {

							public void onClick(DialogInterface dialog,
									int which) {
								doneConfirmed();
							}

						}).setNegativeButton("No", null).show();

		// Intent intent = new Intent(this, RaceReviewActivity.class);
		// // List<String> timesReversed = new
		// ArrayList<String>(currentRace.getTimes());// times held with most
		// recent at position 0 - want to reverse that before saving
		// // Collections.reverse(timesReversed);
		// currentRace.populateIntent(intent);
		// // intent.putExtra(RaceData.TIMES, timesReversed.toArray(new String[]
		// {}));
		//
		// if (currentRace.isChronometerRunning()) {
		// // save times
		//
		// // TODO need to do this as each time is added ...
		// }
		//
		// startActivity(intent);
	}

	/** Check if this device has a camera */
	private boolean checkCameraHardware(Context context) {
		if (context.getPackageManager().hasSystemFeature(
				PackageManager.FEATURE_CAMERA)) {
			// this device has a camera
			return true;
		} else {
			// no camera on this device
			return false;
		}
	}

	// public static boolean isChronometerRunning() {
	// return currentRace.isChronometerRunning();
	// }
	//
	// public static void resetChronometer() {
	// currentRace.reset();
	// }

	/** A safe way to get an instance of the Camera object. */
	public static Camera getCameraInstance() {
		Camera c = null;
		try {
			c = Camera.open(); // attempt to get a Camera instance
		} catch (Exception e) {
			Log.e(TAG, e.getMessage(), e);
			// Camera is not available (in use or does not exist)
		}
		return c; // returns null if camera is unavailable
	}

}
